@@ -1,6 +1,6 @@
 //! Implementation of quadratic integers
 
-use super::{QuadraticOps, QuadraticBase};
+use super::{QuadraticBase, QuadraticOps};
 use core::ops::*;
 use num_integer::Integer;
 use num_traits::{NumRef, One, RefNum, Signed, Zero};
@@ -36,26 +36,14 @@ where
     }
 }
 
-// x/y rounded to the nearest integer. If frac(x/y) = 1/2, then round up.
+// x/y rounded to the nearest integer. If frac(x/y) = 1/2, then round away from zero.
 #[inline]
 fn div_rem_round<T: Integer + Signed + NumRef>(x: T, y: &T) -> (T, T)
 where
     for<'r> &'r T: RefNum<T>,
 {
     let two = T::one() + T::one();
-    let offset = if x.is_negative() {
-        if y.is_negative() {
-            y / two
-        } else {
-            -(y - T::one()) / two
-        }
-    } else {
-        if y.is_negative() {
-            -(y + T::one()) / two
-        } else {
-            y / two
-        }
-    };
+    let offset = y.abs() / two * x.signum();
     let (q, r) = (x + &offset).div_rem(y);
     (q, r - offset)
 }
@@ -63,10 +51,20 @@ where
 /// Underlying representation of a quadratic integer `a + bω` as (a,b), where `ω` is `√D` or `(1+√D)/2`.
 ///
 /// Specifically, `ω=√D` when `D ≡ 2,3 mod 4`, and `ω=(1+√D)/2` when `D ≡ 1 mod 4`. Note that when `ω=(1+√D)/2`,
-/// `ω² = (D-1)/4 + w`
+/// `ω² = (D-1)/4 + w`.
+/// 
+/// The division of two quadratic ints is defined as: calculate a/b = x + yω in the quadratic field, then round x
+/// and y to nearest integer (round half-way cases away from zero, to be consistent with `round()` functions of
+/// primitive types).
 #[derive(Debug, Clone, Copy, PartialEq)]
 pub struct QuadraticIntCoeffs<T>(pub T, pub T);
 
+impl<T> From<(T, T)> for QuadraticIntCoeffs<T> {
+    fn from(v: (T, T)) -> Self {
+        Self(v.0, v.1)
+    }
+}
+
 impl<T: Add<Output = T>> Add<T> for QuadraticIntCoeffs<T> {
     type Output = Self;
     #[inline]
@@ -107,7 +105,8 @@ impl<T: Signed + NumRef> Mul<T> for QuadraticIntCoeffs<T> {
 
 impl<T: Integer + Signed + NumRef> Div<T> for QuadraticIntCoeffs<T>
 where
-    for<'r> &'r T: RefNum<T> {
+    for<'r> &'r T: RefNum<T>,
+{
     type Output = Self;
     #[inline]
     fn div(self, rhs: T) -> Self::Output {
@@ -116,7 +115,8 @@ where
 }
 impl<T: Integer + Signed + NumRef + Clone> Rem<T> for QuadraticIntCoeffs<T>
 where
-    for<'r> &'r T: RefNum<T> {
+    for<'r> &'r T: RefNum<T>,
+{
     type Output = Self;
     #[inline]
     fn rem(self, rhs: T) -> Self::Output {
@@ -124,7 +124,6 @@ where
     }
 }
 
-
 impl<T: Integer + Signed + NumRef> QuadraticIntCoeffs<T>
 where
     for<'r> &'r T: RefNum<T>,
@@ -132,7 +131,7 @@ where
     /// Get the norm of the quadratic integer (taking reference)
     fn norm_ref(&self, discr: &T) -> T {
         if discr.is_zero() {
-            return &self.0 * &self.0
+            return &self.0 * &self.0;
         }
 
         match mod4d2(discr) {
@@ -207,7 +206,7 @@ where
     #[inline]
     fn conj(self, discr: &T) -> Self {
         if discr.is_zero() {
-            return self
+            return self;
         }
 
         match mod4d2(discr) {
@@ -230,44 +229,82 @@ where
 ///
 /// The different between [QuadraticInt] and [QuadraticSurd][crate::QuadraticSurd] is that the operations for the
 /// latter will be in normal fields of real numbers or complex numbers, while the operations
-/// for the former will be in the Quadratic Field (specifically in the quadratic integer ring ℤ\[ω\])
-/// The arithmetic operations can only be performed between the integers with the same base.
+/// for the former will be in the quadratic field (specifically in the quadratic integer ring ℤ\[ω\]).
+/// Therefore, the arithmetic operations between [QuadraticInt]s can only be performed with equivalent bases (e.g. `√18=3√2` is compatible with `√2`).
+/// However for [QuadraticSurd][crate::QuadraticSurd], you can mix the number with different bases if they are both pure quadratic numbers,
+/// such as `√2 * √3 = √6`.
 #[derive(Debug, Clone, Copy)]
 pub struct QuadraticInt<T> {
     coeffs: QuadraticIntCoeffs<T>, // (a, b), b = 0 is ensured when D = 0
-    discr: T, // D
+    discr: T,                      // D, D mod 4 != 0 is ensured when D != 0
 }
 
-impl<T: Integer + Signed + NumRef> QuadraticInt<T>
+impl<T: QuadraticBase> QuadraticInt<T>
 where
     for<'r> &'r T: RefNum<T>,
 {
-    /// Create a quadratic integer `a + bω`, where `ω` is `√r` or `(1+√r)/2`.
-    /// Note that r must be not divisible by 4 (to be square free), otherwise the factor 4
-    /// will be extracted from r to b.
+    /// Create a quadratic integer from `a + b√r`, where a, b, r are all integers
+    #[inline]
     pub fn new(a: T, b: T, r: T) -> Self {
-        if b.is_zero() || r.is_zero() {
-            return Self { coeffs: QuadraticIntCoeffs(a, T::zero()), discr: T::zero() }
-        }
-        #[cfg(not(feature = "complex"))]
-        if r.is_negative() {
-            panic!("Negative root is not supported without the `complex` feature");
+        let (mut b, mut r) = (b, r);
+
+        // remove factor 4
+        if r.is_positive() {
+            while (&r % four::<T>()).is_zero() {
+                r = r / four::<T>();
+                b = &b + &b;
+            }
         }
 
-        let mut b = b;
-        let mut discr = r;
-        while (&discr % four::<T>()).is_zero() {
-            discr = discr / four::<T>();
-            b = b * (T::one() + T::one());
-        }
-        Self {
-            coeffs: QuadraticIntCoeffs(a, b),
-            discr,
+        match mod4d2(&r) {
+            0 => {
+                let (y, x) = (&b + &b, a - b);
+                Self::from_coeffs(QuadraticIntCoeffs(x, y), r)
+            },
+            1 => Self::from_coeffs(QuadraticIntCoeffs(a, b), r),
+            _ => unreachable!()
         }
     }
-    #[inline]
-    pub fn from_coeffs(coeffs: QuadraticIntCoeffs<T>, r: T) -> Self {
-        Self::new(coeffs.0, coeffs.1, r)
+    /// Create a quadratic integer `a + bω`, where `ω` is `√D` or `(1+√D)/2`.
+    /// 
+    /// # Panics
+    /// - If the discriminant is negative and feature "complex" is not enabled
+    /// - The discriminant is congruent to 0 modulo 4
+    pub fn from_coeffs(coeffs: QuadraticIntCoeffs<T>, discr: T) -> Self {
+        #[cfg(not(feature = "complex"))]
+        if discr.is_negative() {
+            panic!("Negative root is not supported without the `complex` feature");
+        }
+        if (&discr % four::<T>()).is_zero() {
+            panic!("The discriminant must not be 0 modulo 4!")
+        }
+
+        let QuadraticIntCoeffs(a, b) = coeffs;
+
+        // test if the root is a square number
+        let root = discr.abs().sqrt();
+        let (a, b, r) = if &root * &root == discr {
+            if discr.is_negative() {
+                (a, b * root, -T::one())
+            } else {
+                (a + b * root, T::zero(), T::zero())
+            }
+        } else {
+            (a, b, discr)
+        };
+
+        // shortcut for rational number
+        if b.is_zero() || r.is_zero() {
+            return Self {
+                coeffs: QuadraticIntCoeffs(a, T::zero()),
+                discr: T::zero(),
+            };
+        }
+
+        Self {
+            coeffs: QuadraticIntCoeffs(a, b),
+            discr: r,
+        }
     }
 
     #[inline]
@@ -285,12 +322,32 @@ where
     pub fn is_rational(&self) -> bool {
         self.coeffs.1.is_zero()
     }
+    #[inline]
+    pub fn is_pure(&self) -> bool {
+        match mod4d2(&self.discr) {
+            0 => (&self.coeffs.0 + &self.coeffs.0 + &self.coeffs.1).is_zero(),
+            1 => self.coeffs.1.is_zero(),
+            _ => unreachable!(),
+        }
+    }
 
     /// Get the fundamental unit of the quadratic field ℚ[√d]
     fn unit(d: T) -> Self {
         // REF: http://www.numbertheory.org/gnubc/unit
+        // REF: https://people.reed.edu/~jerry/361/lectures/rqunits.pdf
         unimplemented!()
     }
+    
+    /// Convert `a + bω` to `(x + y√D)/2`
+    #[inline]
+    fn parts_doubled(&self) -> (T, T) {
+        let QuadraticIntCoeffs(a, b) = &self.coeffs;
+        match mod4d2(&self.discr) {
+            0 => (a + a + b, b.clone()),
+            1 => (a + a, b + b),
+            _ => unreachable!()
+        }
+    }
 }
 
 impl<T: QuadraticBase> QuadraticInt<T>
@@ -306,16 +363,21 @@ where
 
         let (quo, rem) = self.discr.div_rem(&hint);
         if rem.is_zero() {
-            // if hint is actually a factor
+            // if hint is actually a factor ...
             let root = hint.clone().sqrt();
             if &root * &root == hint {
-                // if hint is a square number, then remove the hint factor
-                // note that r ≡ r/a^2 mod 4 (given a is odd)
-                return Self::new(
-                    self.coeffs.0,
-                    self.coeffs.1 * root,
-                    quo
-                );
+                // and if hint is a square number, then remove the hint factor.
+                // note that r ≡ r/a^2 mod 4 (given a is odd), so reducing is feasible
+                return match mod4d2(&self.discr) {
+                    0 => {
+                        // ω_{a*a*r} = (1 + a√r)/2 = a(1 + √r)/2 - (a-1)/2 = a*ω_{r} - (a-1)/2
+                        let scale = (&root - T::one()) / (T::one() + T::one());
+                        Self::from_coeffs(QuadraticIntCoeffs(self.coeffs.0 - &self.coeffs.1 * scale, self.coeffs.1 * root), quo)
+                    },
+                    1 => Self::from_coeffs(QuadraticIntCoeffs(self.coeffs.0, self.coeffs.1 * root), quo),
+                    _ => unreachable!()
+                }
+                
             }
         }
 
@@ -326,7 +388,10 @@ where
 impl<T: Neg<Output = T>> Neg for QuadraticInt<T> {
     type Output = Self;
     fn neg(self) -> Self::Output {
-        Self{ coeffs: QuadraticIntCoeffs(-self.coeffs.0, -self.coeffs.1), discr: self.discr }
+        Self {
+            coeffs: QuadraticIntCoeffs(-self.coeffs.0, -self.coeffs.1),
+            discr: self.discr,
+        }
     }
 }
 
@@ -405,7 +470,8 @@ where
 
 impl<T: QuadraticBase> PartialEq for QuadraticInt<T>
 where
-    for<'r> &'r T: RefNum<T>, {
+    for<'r> &'r T: RefNum<T>,
+{
     fn eq(&self, other: &Self) -> bool {
         // shortcuts
         if self.discr == other.discr {
@@ -415,20 +481,22 @@ where
             return false;
         }
 
-        let QuadraticIntCoeffs(la, lb) = &self.coeffs;
-        let QuadraticIntCoeffs(ra, rb) = &other.coeffs;
-        la == ra && lb * lb * &self.discr == rb * rb * &other.discr
+        let (la, lb) = self.parts_doubled();
+        let (ra, rb) = other.parts_doubled();
+
+        la == ra && &lb * &lb * &self.discr == &rb * &rb * &other.discr
     }
 }
 
-impl<T: QuadraticBase> Eq for QuadraticInt<T>
-where
-    for<'r> &'r T: RefNum<T>, { }
+impl<T: QuadraticBase> Eq for QuadraticInt<T> where for<'r> &'r T: RefNum<T> {}
 
 impl<T: Add<Output = T>> Add<T> for QuadraticInt<T> {
     type Output = Self;
     fn add(self, rhs: T) -> Self::Output {
-        Self { coeffs: self.coeffs + rhs, discr: self.discr }
+        Self {
+            coeffs: self.coeffs + rhs,
+            discr: self.discr,
+        }
     }
 }
 impl<T: QuadraticBase> Add for QuadraticInt<T>
@@ -453,7 +521,10 @@ where
 impl<T: Sub<Output = T>> Sub<T> for QuadraticInt<T> {
     type Output = Self;
     fn sub(self, rhs: T) -> Self::Output {
-        Self { coeffs: self.coeffs - rhs, discr: self.discr }
+        Self {
+            coeffs: self.coeffs - rhs,
+            discr: self.discr,
+        }
     }
 }
 impl<T: QuadraticBase> Sub for QuadraticInt<T>
@@ -485,7 +556,10 @@ where
         if rhs.is_zero() {
             Self::zero()
         } else {
-            Self { coeffs: self.coeffs * rhs, discr: self.discr }
+            Self {
+                coeffs: self.coeffs * rhs,
+                discr: self.discr,
+            }
         }
     }
 }
@@ -502,10 +576,15 @@ where
         if self.is_rational() {
             return rhs * self.coeffs.0;
         }
-        // TODO: add is_pure check
+        // XXX: currently, multiplication (and division) of pure quadratic numbers is not allowed here, because
+        // it might lead to non-representable results. For example, `ω(2) * (ω(5)-1/2) = √2 * √5/2 = √10/2`,
+        // the result is not a quadratic integer
 
         let (lhs, rhs) = reduce_bin_op_unwrap(self, rhs);
-        Self::from_coeffs(QuadraticOps::mul(lhs.coeffs, rhs.coeffs, &lhs.discr), rhs.discr)
+        Self::from_coeffs(
+            QuadraticOps::mul(lhs.coeffs, rhs.coeffs, &lhs.discr),
+            rhs.discr,
+        )
     }
 }
 
@@ -516,7 +595,10 @@ where
     type Output = Self;
     #[inline]
     fn div(self, rhs: T) -> Self::Output {
-        Self { coeffs: self.coeffs / rhs, discr: self.discr }
+        Self {
+            coeffs: self.coeffs / rhs,
+            discr: self.discr,
+        }
     }
 }
 impl<T: QuadraticBase> Div for QuadraticInt<T>
@@ -529,13 +611,18 @@ where
         if rhs.is_rational() {
             return self / rhs.coeffs.0;
         }
-        // TODO: add is_pure check
 
         if self.is_rational() {
-            Self::from_coeffs(QuadraticOps::div(self.coeffs, rhs.coeffs, &rhs.discr), rhs.discr)
+            Self::from_coeffs(
+                QuadraticOps::div(self.coeffs, rhs.coeffs, &rhs.discr),
+                rhs.discr,
+            )
         } else {
             let (lhs, rhs) = reduce_bin_op_unwrap(self, rhs);
-            Self::from_coeffs(QuadraticOps::div(lhs.coeffs, rhs.coeffs, &lhs.discr), rhs.discr)
+            Self::from_coeffs(
+                QuadraticOps::div(lhs.coeffs, rhs.coeffs, &lhs.discr),
+                rhs.discr,
+            )
         }
     }
 }
@@ -547,7 +634,10 @@ where
     type Output = Self;
     #[inline]
     fn rem(self, rhs: T) -> Self::Output {
-        Self { coeffs: self.coeffs % rhs, discr: self.discr }
+        Self {
+            coeffs: self.coeffs % rhs,
+            discr: self.discr,
+        }
     }
 }
 impl<T: QuadraticBase> Rem for QuadraticInt<T>
@@ -560,7 +650,6 @@ where
         if rhs.is_rational() {
             return self / rhs.coeffs.0;
         }
-        // TODO: add is_pure check
 
         let (lhs, rhs) = reduce_bin_op_unwrap(self, rhs);
         Self::from_coeffs(lhs.coeffs.div_rem(rhs.coeffs, &lhs.discr).1, rhs.discr)
@@ -569,9 +658,13 @@ where
 
 impl<T: QuadraticBase> Zero for QuadraticInt<T>
 where
-    for<'r> &'r T: RefNum<T>, {
+    for<'r> &'r T: RefNum<T>,
+{
     fn zero() -> Self {
-        Self { coeffs: QuadraticIntCoeffs(T::zero(), T::zero()), discr: T::zero() }
+        Self {
+            coeffs: QuadraticIntCoeffs(T::zero(), T::zero()),
+            discr: T::zero(),
+        }
     }
 
     fn is_zero(&self) -> bool {
@@ -581,9 +674,13 @@ where
 
 impl<T: QuadraticBase> One for QuadraticInt<T>
 where
-    for<'r> &'r T: RefNum<T>, {
+    for<'r> &'r T: RefNum<T>,
+{
     fn one() -> Self {
-        Self { coeffs: QuadraticIntCoeffs(T::one(), T::zero()), discr: T::zero() }
+        Self {
+            coeffs: QuadraticIntCoeffs(T::one(), T::zero()),
+            discr: T::zero(),
+        }
     }
 }
 
@@ -610,7 +707,8 @@ mod complex {
 
     impl<T: Add<Output = T>> GaussianInt<T>
     where
-        for<'r> &'r T: RefNum<T>, {
+        for<'r> &'r T: RefNum<T>,
+    {
         pub fn norm_ref(&self) -> T {
             let QuadraticIntCoeffs(a, b) = &self.0;
             a * a + b * b
@@ -663,7 +761,7 @@ mod complex {
             Self(QuadraticOps::div(self.0, rhs.0, &-T::one()))
         }
     }
-    
+
     impl<T: Integer + Signed + NumRef> Rem for GaussianInt<T>
     where
         for<'r> &'r T: RefNum<T>,
@@ -675,7 +773,7 @@ mod complex {
         }
     }
 
-    // TODO: implement num_integer::Integer for GaussianInt, especially is_even and is_odd can be derived from
+    // TODO (v0.3.1): implement num_integer::Integer for GaussianInt, especially is_even and is_odd can be derived from
     // https://crates.io/crates/gaussiant
 }
 
@@ -692,26 +790,33 @@ mod tests {
             // x, y, (x/y), (x%y)
             (-4, 4, -1, 0),
             (-3, 4, -1, 1),
-            (-2, 4, 0, -2),
+            (-2, 4, -1, 2),
             (-1, 4, 0, -1),
             (0, 4, 0, 0),
             (1, 4, 0, 1),
             (2, 4, 1, -2),
             (3, 4, 1, -1),
             (4, 4, 1, 0),
-
             (-4, -4, 1, 0),
             (-3, -4, 1, 1),
             (-2, -4, 1, 2),
             (-1, -4, 0, -1),
             (0, -4, 0, 0),
             (1, -4, 0, 1),
-            (2, -4, 0, 2),
+            (2, -4, -1, -2),
             (3, -4, -1, -1),
             (4, -4, -1, 0),
         ];
         for (x, y, q, r) in CASES {
-            assert_eq!(div_rem_round(x, &y), (q, r), "{} / {} != ({}, {})", x, y, q, r);
+            assert_eq!(
+                div_rem_round(x, &y),
+                (q, r),
+                "{} / {} != ({}, {})",
+                x,
+                y,
+                q,
+                r
+            );
         }
     }
 
@@ -746,7 +851,7 @@ mod tests {
         #[cfg(feature = "complex")]
         {
             // unit complex number tests
-            let zero = QuadraticInt::new(0, 0, 2);
+            let zero = QuadraticInt::new(0, 0, -1);
             let e1 = QuadraticInt::new(1, 0, -1);
             let e2 = QuadraticInt::new(0, 1, -1);
             let e3 = QuadraticInt::new(-1, 0, -1);
@@ -792,8 +897,8 @@ mod tests {
         assert_eq!(a + b, QuadraticInt::new(-1, 2, 2));
         assert_eq!(a - b, QuadraticInt::new(3, 4, 2));
         assert_eq!(a * b, QuadraticInt::new(-8, -7, 2));
-        assert_eq!(a / b, QuadraticInt::new(2, -2, 2));
-        assert_eq!(a % b, QuadraticInt::new(1, 1, 2));
+        assert_eq!(a / b, QuadraticInt::new(2, -3, 2));
+        assert_eq!(a % b, QuadraticInt::new(-1, -1, 2));
 
         assert_eq!(b + c, QuadraticInt::new(-3, 1, 2));
         assert_eq!(b - c, QuadraticInt::new(-1, -3, 2));
@@ -850,18 +955,18 @@ mod tests {
     #[cfg(feature = "complex")]
     fn test_eisenstein() {
         // test Eisenstein integers
-        let q12 = QuadraticInt::new(1, 2, -3);
-        let qm12 = QuadraticInt::new(-1, 2, -3);
-        let q3m2 = QuadraticInt::new(3, -2, -3);
-        let q23 = QuadraticInt::new(2, 3, -3);
-        let qm23 = QuadraticInt::new(-2, 3, -3);
-        let q5m3 = QuadraticInt::new(5, -3, -3);
+        let q12 =  QuadraticInt::from_coeffs((1, 2).into(), -3); // 2 + √-3
+        let qm12 = QuadraticInt::from_coeffs((-1, 2).into(), -3); // √-3
+        let q3m2 = QuadraticInt::from_coeffs((3, -2).into(), -3); // 2 - √-3
+        let q23 =  QuadraticInt::from_coeffs((2, 3).into(), -3);
+        let qm23 = QuadraticInt::from_coeffs((-2, 3).into(), -3);
+        let q5m3 = QuadraticInt::from_coeffs((5, -3).into(), -3);
 
         assert_eq!(q12.conj(), q3m2);
         assert_eq!(q23.conj(), q5m3);
         assert_eq!(q12.norm(), 7);
         assert_eq!(q23.norm(), 19);
-        assert_eq!(q12 * q12, QuadraticInt::new(-3, 8, -3));
+        assert_eq!(q12 * q12, QuadraticInt::from_coeffs((-3, 8).into(), -3));
 
         for &v1 in &[q12, qm12, q3m2, q23, qm23, q5m3] {
             for &v2 in &[q12, qm12, q3m2, q23, qm23, q5m3] {
