@@ -1,14 +1,14 @@
 //! Implementation of qudratic irrational numbers
 
 use super::{QuadraticBase, QuadraticOps};
-use crate::cont_frac::simple::ContinuedFraction;
+use crate::cont_frac::ContinuedFraction;
 use crate::traits::{
     Approximation, Computable, FromSqrt, FromSqrtError, SqrtErrorKind, WithSigned, WithUnsigned,
 };
 #[cfg(feature = "complex")]
-use core::convert::TryFrom;
-#[cfg(feature = "complex")]
 use crate::GaussianInt;
+use crate::QuadraticInt;
+use core::convert::TryFrom;
 use core::ops::{Add, AddAssign, Div, Mul, Neg, Sub};
 use num_integer::{sqrt, Integer};
 use num_traits::{
@@ -26,6 +26,12 @@ use num_rational::Ratio;
 #[derive(Clone, Copy, Debug, Hash, PartialEq, Eq)]
 pub struct QuadraticSurdCoeffs<T>(pub T, pub T, pub T);
 
+impl<T> From<(T, T, T)> for QuadraticSurdCoeffs<T> {
+    fn from(v: (T, T, T)) -> Self {
+        Self(v.0, v.1, v.2)
+    }
+}
+
 impl<T: QuadraticBase> Add for QuadraticSurdCoeffs<T>
 where
     for<'r> &'r T: RefNum<T>,
@@ -172,6 +178,18 @@ impl<T: QuadraticBase> Div<T> for QuadraticSurdCoeffs<T> {
 /// A quadratic number represented as `(a + b*√r) / c`.
 /// If the support for complex number is enabled, then this struct can represent
 /// any quadratic integers.
+///
+/// # About the `complex` feature
+///
+/// Whether enabling the `complex` feature of the crate will lead to some behavior changes
+/// of this struct (in a compatible way), some important ones are listed below:
+/// - The [new()][QuadraticSurd::new] constructor will panic if `complex` is disabled and the given root is negative
+/// - Rounding functions ([fract()][QuadraticSurd::fract], [trunc()][QuadraticSurd::trunc], ..) will round to Gaussian
+///   integers instead of rational integers if `complex` is enabled.
+/// - Rational approximation ([approximated()][QuadraticSurd::approximated]) will panic if `complex` is enabled and
+///   the quadratic surd is complex
+/// - [QuadraticSurd::from_sqrt()] won't fail as [SqrtErrorKind::Complex] for negative input if `complex` is enabled
+///
 #[derive(Hash, Clone, Debug, Copy)]
 pub struct QuadraticSurd<T> {
     coeffs: QuadraticSurdCoeffs<T>, // when reduced, coeffs.1 is zero if the surd is rational, coeffs.2 is always positive
@@ -209,8 +227,8 @@ impl<T: Integer> QuadraticSurd<T> {
 
     /// Determine if the surd is a quadratic integer
     #[inline]
-    fn is_quadint(&self) -> bool {
-        unimplemented!()
+    pub fn is_quadint(&self) -> bool {
+        self.coeffs.2.is_one()
     }
 
     /// Determine if the surd is a rational number
@@ -232,6 +250,7 @@ impl<T: Integer> QuadraticSurd<T> {
         self.coeffs.0.is_zero() && !self.coeffs.1.is_zero()
     }
 
+    /// Panic when the number is complex and the feature "complex" is disabled
     #[inline(always)]
     fn panic_if_complex(&self) {
         // only need to check when we allow construction of complex surd number
@@ -277,16 +296,10 @@ where
             panic!("denominator is zero");
         }
 
-        // ensure b, r are zeros at the same time
-        if self.coeffs.1.is_zero() {
-            self.discr = T::zero();
-        }
-
-        // test if the surd is rational
+        // test and reduce if the surd is rational
         let root = sqrt(self.discr.abs());
         if &root * &root == self.discr {
             if self.discr.is_negative() {
-                self.coeffs.0 = T::zero();
                 self.coeffs.1 = &self.coeffs.1 * root;
                 self.discr = -T::one();
             } else {
@@ -296,21 +309,31 @@ where
             }
         }
 
-        // reduce common divisor
-        let mut g_ac = self.coeffs.0.gcd(&self.coeffs.2);
-        let g_acr = (&g_ac * &g_ac).gcd(&self.discr); // test if squared factor of c can divide r
-        let groot = sqrt(g_acr.clone());
-        if &groot * &groot == g_acr {
-            self.discr = &self.discr / &g_acr;
-            self.coeffs.0 = &self.coeffs.0 / &groot;
-            self.coeffs.2 = &self.coeffs.2 / &groot;
-            g_ac = &g_ac / groot;
-        }
+        if self.coeffs.1.is_zero() || self.discr.is_zero() {
+            // shortcut if the surd is rational
+            self.discr = T::zero();
+            self.coeffs.1 = T::zero();
 
-        let g_abc = g_ac.gcd(&self.coeffs.1);
-        self.coeffs.0 = &self.coeffs.0 / &g_abc;
-        self.coeffs.1 = &self.coeffs.1 / &g_abc;
-        self.coeffs.2 = &self.coeffs.2 / g_abc;
+            let g_ac = self.coeffs.0.gcd(&self.coeffs.2);
+            self.coeffs.0 = &self.coeffs.0 / &g_ac;
+            self.coeffs.2 = &self.coeffs.2 / g_ac;
+        } else {
+            // reduce common divisor
+            let mut g_ac = self.coeffs.0.gcd(&self.coeffs.2);
+            let g_acr = (&g_ac * &g_ac).gcd(&self.discr); // test if squared factor of c can divide r
+            let groot = sqrt(g_acr.clone());
+            if &groot * &groot == g_acr {
+                self.discr = &self.discr / &g_acr;
+                self.coeffs.0 = &self.coeffs.0 / &groot;
+                self.coeffs.2 = &self.coeffs.2 / &groot;
+                g_ac = &g_ac / groot;
+            }
+
+            let g_abc = g_ac.gcd(&self.coeffs.1);
+            self.coeffs.0 = &self.coeffs.0 / &g_abc;
+            self.coeffs.1 = &self.coeffs.1 / &g_abc;
+            self.coeffs.2 = &self.coeffs.2 / g_abc;
+        }
 
         // keep denom positive
         if self.coeffs.2 < T::zero() {
@@ -454,23 +477,22 @@ where
         self.clone().conj()
     }
 
-    /// Round the surd toward zero.
+    /// Round the surd toward zero. The result will be an integer if the surd is real,
+    /// or a Gaussian integer if the surd is complex
     ///
     /// # Panics
     /// if the number is complex, when the `complex` feature is not enabled
     pub fn trunc(self) -> Self {
-        self.panic_if_complex();
+        let QuadraticSurdCoeffs(a, b, c) = self.coeffs;
+        let bsign = b.signum();
 
-        // TODO: ensure the algorithm is correct, and add tests
-        // TODO: trunc to Gaussian integer if `complex` is enabled
-        let bneg = self.coeffs.1.is_negative();
-        let br = sqrt(&self.coeffs.1 * &self.coeffs.1 * self.discr);
-        let num = if bneg {
-            self.coeffs.0 - br
+        if self.discr.is_negative() {
+            let br = bsign * sqrt(&b * &b * -self.discr);
+            Self::from_coeffs(QuadraticSurdCoeffs(a / &c, br / c, T::one()), -T::one())
         } else {
-            self.coeffs.0 + br
-        };
-        return Self::from(num / self.coeffs.2);
+            let br = bsign * sqrt(&b * &b * self.discr);
+            return Self::from((a + br) / c);
+        }
     }
 
     /// [QuadraticSurd::trunc()] with reference
@@ -482,11 +504,9 @@ where
     /// Get the fractional part of the surd, ensuring `self.trunc() + self.fract() == self`
     ///
     /// # Panics
-    /// if the number is complex, when the `complex` feature is not enabled
+    /// If the square root base is negative and not -1 (the result won't be representable by this struct)
     #[inline]
     pub fn fract(self) -> Self {
-        self.panic_if_complex();
-
         let trunc = self.trunc_ref();
         self - trunc
     }
@@ -514,8 +534,6 @@ where
         Self::from(self.coeffs.2.clone())
     }
 
-    // TODO: add ceil(), floor(), round()
-
     /// Converts to an integer, rounding towards zero
     ///
     /// # Panics
@@ -535,16 +553,28 @@ where
     #[inline]
     #[cfg(feature = "complex")]
     pub fn to_gaussint(&self) -> Approximation<GaussianInt<T>> {
-        self.panic_if_complex();
-
         if self.is_gaussint() {
-            Approximation::Exact(GaussianInt::new(self.coeffs.0.clone(), self.coeffs.1.clone()))
+            Approximation::Exact(GaussianInt::new(
+                self.coeffs.0.clone(),
+                self.coeffs.1.clone(),
+            ))
         } else {
             let trunc = self.trunc_ref();
             Approximation::Approximated(GaussianInt::new(trunc.coeffs.0, trunc.coeffs.1))
         }
     }
 
+    /// Converts to an quadratic integer, rounding towards zero
+    #[inline]
+    pub fn to_quadint(&self) -> Approximation<QuadraticInt<T>> {
+        let QuadraticSurdCoeffs(a, b, c) = &self.coeffs;
+        if self.is_quadint() {
+            Approximation::Exact(QuadraticInt::new(a.clone(), b.clone(), self.discr.clone()))
+        } else {
+            Approximation::Approximated(QuadraticInt::new(a / c, b / c, self.discr.clone()))
+        }
+    }
+
     /// Converts to a rational, rounding square root towards zero
     ///
     /// # Panics
@@ -593,6 +623,11 @@ where
         self.clone().floor()
     }
 
+    fn ceil(self) -> Self { unimplemented!() }
+
+    /// Returns the nearest integer to a number. Round half-way cases away from zero?.
+    fn round(self) -> Self { unimplemented!() }
+
     /// Test if the surd number is positive
     ///
     /// # Panics
@@ -846,23 +881,9 @@ where
     ContinuedFraction::new(a_coeffs, p_coeffs, neg)
 }
 
-#[cfg(not(feature = "complex"))]
-impl<T: QuadraticBase + WithUnsigned<Unsigned = U> + AddAssign, U> From<QuadraticSurd<T>>
-    for ContinuedFraction<U>
-where
-    for<'r> &'r T: RefNum<T>,
-{
-    fn from(s: QuadraticSurd<T>) -> Self {
-        if s.is_negative() {
-            quadsurd_to_contfrac(-s.coeffs.0, -s.coeffs.1, s.coeffs.2, s.discr, true)
-        } else {
-            quadsurd_to_contfrac(s.coeffs.0, s.coeffs.1, s.coeffs.2, s.discr, false)
-        }
-    }
-}
-
-// TODO: convert to continued fraction based on gaussian integers if `complex` is enabled
-#[cfg(feature = "complex")]
+// Although conversion to continued fraction won't fail if complex number is disabled,
+// we have to disable From<> to make sure that adding the feature 'complex' won't result
+// in break change
 impl<T: QuadraticBase + WithUnsigned<Unsigned = U> + AddAssign, U> TryFrom<QuadraticSurd<T>>
     for ContinuedFraction<U>
 where
@@ -891,6 +912,9 @@ where
     }
 }
 
+// TODO: support convert to hurwitz continued fraction based on gaussian integers if `complex` is enabled
+//       this should be implemented as From<QuadraticSurd<T>> for HurwitzContinuedFraction (instead of TryFrom)
+
 impl<
         T: QuadraticBase + CheckedAdd + AddAssign + WithUnsigned<Unsigned = U>,
         U: Integer + Clone + CheckedAdd + CheckedMul + WithSigned<Signed = T>,
@@ -898,12 +922,6 @@ impl<
 where
     for<'r> &'r T: RefNum<T>,
 {
-    #[cfg(not(feature = "complex"))]
-    fn approximated(&self, limit: &T) -> Approximation<Ratio<T>> {
-        ContinuedFraction::<U>::from(self.clone()).approximated(limit)
-    }
-
-    #[cfg(feature = "complex")]
     fn approximated(&self, limit: &T) -> Approximation<Ratio<T>> {
         ContinuedFraction::<U>::try_from(self.clone())
             .expect("only real numbers can be approximated by a rational number")
@@ -911,48 +929,74 @@ where
     }
 }
 
-impl<T: Integer + Signed + fmt::Display> fmt::Display for QuadraticSurd<T> {
+impl<T: Integer + Signed + fmt::Display + Clone> fmt::Display for QuadraticSurd<T> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let QuadraticSurdCoeffs(a, b, c) = &self.coeffs;
-        if f.alternate() && self.discr == -T::one() {
-            // print √-1 as i if alternate flag is set
+        if f.alternate() && self.discr.is_negative() {
+            // print √-1 as i, √-5 as √5i if alternate flag is set
+            // XXX: refactor this function
+            let r = -self.discr.clone();
             match (
                 a.is_zero(),
                 b.is_zero(),
                 b.is_one(),
                 b == &-T::one(),
                 c.is_one(),
+                r.is_one(),
             ) {
-                (true, true, _, _, _) => write!(f, "0"),
-                (true, false, true, _, true) => write!(f, "i"),
-                (true, false, true, _, false) => write!(f, "i/{}", c),
-                (true, false, false, true, true) => write!(f, "-i"),
-                (true, false, false, true, false) => write!(f, "-i/{}", c),
-                (true, false, false, false, true) => write!(f, "{}i", b),
-                (true, false, false, false, false) => write!(f, "{}i/{}", b, c),
-                (false, true, _, _, true) => write!(f, "{}", a),
-                (false, true, _, _, false) => write!(f, "{}/{}", a, c),
-                (false, false, true, _, true) => write!(f, "{}+i", a),
-                (false, false, false, true, true) => write!(f, "{}-i", a),
-                (false, false, false, false, true) => {
+                (true, true, _, _, _, _) => write!(f, "0"),
+                (true, false, true, _, true, true) => write!(f, "i"),
+                (true, false, true, _, true, false) => write!(f, "√{}i", r),
+                (true, false, true, _, false, true) => write!(f, "i/{}", c),
+                (true, false, true, _, false, false) => write!(f, "√{}i/{}", r, c),
+                (true, false, false, true, true, true) => write!(f, "-i"),
+                (true, false, false, true, true, false) => write!(f, "-√{}i", r),
+                (true, false, false, true, false, true) => write!(f, "-i/{}", c),
+                (true, false, false, true, false, false) => write!(f, "-√{}i/{}", r, c),
+                (true, false, false, false, true, true) => write!(f, "{}i", b),
+                (true, false, false, false, true, false) => write!(f, "{}√{}i", b, r),
+                (true, false, false, false, false, true) => write!(f, "{}i/{}", b, c),
+                (true, false, false, false, false, false) => write!(f, "{}√{}i/{}", b, r, c),
+                (false, true, _, _, true, _) => write!(f, "{}", a),
+                (false, true, _, _, false, _) => write!(f, "{}/{}", a, c),
+                (false, false, true, _, true, true) => write!(f, "{}+i", a),
+                (false, false, true, _, true, false) => write!(f, "{}+√{}i", a, r),
+                (false, false, false, true, true, true) => write!(f, "{}-i", a),
+                (false, false, false, true, true, false) => write!(f, "{}-√{}i", a, r),
+                (false, false, false, false, true, true) => {
                     if b.is_negative() {
                         write!(f, "{}{}i", a, b)
                     } else {
                         write!(f, "{}+{}i", a, b)
                     }
+                },
+                (false, false, false, false, true, false) => {
+                    if b.is_negative() {
+                        write!(f, "{}{}√{}i", a, b, r)
+                    } else {
+                        write!(f, "{}+{}√{}i", a, b, r)
+                    }
                 }
-                (false, false, true, _, false) => write!(f, "({}+i)/{}", a, c),
-                (false, false, false, true, false) => write!(f, "({}-i)/{}", a, c),
-                (false, false, false, false, false) => {
+                (false, false, true, _, false, true) => write!(f, "({}+i)/{}", a, c),
+                (false, false, true, _, false, false) => write!(f, "({}+√{}i)/{}", a, r, c),
+                (false, false, false, true, false, true) => write!(f, "({}-i)/{}", a, c),
+                (false, false, false, true, false, false) => write!(f, "({}-√{}i)/{}", a, r, c),
+                (false, false, false, false, false, true) => {
                     if b.is_negative() {
                         write!(f, "({}{}i)/{}", a, b, c)
                     } else {
                         write!(f, "({}+{}i)/{}", a, b, c)
                     }
+                },
+                (false, false, false, false, false, false) => {
+                    if b.is_negative() {
+                        write!(f, "({}{}√{}i)/{}", a, b, r, c)
+                    } else {
+                        write!(f, "({}+{}√{}i)/{}", a, b, r, c)
+                    }
                 }
             }
         } else {
-            // TODO: print √-5 as √5i if alternative flag is on. Implement this after refactor this function
             match (
                 a.is_zero(),
                 b.is_zero(),
@@ -1148,7 +1192,7 @@ where
         }
         if self.is_pure() && rhs.is_pure() {
             let gcd = self.discr.gcd(&rhs.discr);
-            let discr = self.discr * rhs.discr / (&gcd * &gcd);
+            let discr = (self.discr / &gcd) * (rhs.discr / &gcd);
             return Self::new(
                 T::zero(),
                 self.coeffs.1 * rhs.coeffs.1 * gcd,
@@ -1182,12 +1226,12 @@ where
         }
         if self.is_pure() && rhs.is_pure() {
             let gcd = self.discr.gcd(&rhs.discr);
-            let discr = self.discr * &rhs.discr / (&gcd * &gcd);
+            let (ld, rd) = (self.discr / &gcd, rhs.discr / gcd);
             return Self::new(
                 T::zero(),
-                self.coeffs.1 * rhs.coeffs.2 * gcd,
-                self.coeffs.2 * rhs.coeffs.1 * rhs.discr,
-                discr,
+                self.coeffs.1 * rhs.coeffs.2,
+                self.coeffs.2 * rhs.coeffs.1 * &rd,
+                ld * rd,
             );
         }
 
@@ -1364,10 +1408,8 @@ where
             });
         }
         if target.is_integer() {
-            return Self::from_sqrt(target.to_integer()).map_err(|e| FromSqrtError {
-                data: Ratio::from(e.data),
-                kind: e.kind,
-            });
+            let (num, _) = target.into();
+            return Ok(QuadraticSurd::new(T::zero(), T::one(), T::one(), num));
         }
 
         match target.numer().checked_mul(target.denom()) {
@@ -1389,6 +1431,7 @@ impl<T: QuadraticBase + CheckedMul> FromSqrt<QuadraticSurd<T>> for QuadraticSurd
 where
     for<'r> &'r T: RefNum<T>,
 {
+    /// The real part of the output is ensured to be non-negative (if successful).
     #[inline]
     fn from_sqrt(target: QuadraticSurd<T>) -> Result<Self, FromSqrtError<QuadraticSurd<T>>> {
         #[cfg(not(feature = "complex"))]
@@ -1421,44 +1464,62 @@ where
         let y = Ratio::new(b, c);
         let x_y = x / &y;
         let delta2 = &x_y * &x_y - &target.discr;
-        if delta2.is_negative() {
-            // TODO: How to support complex quadratic surd here?
-            return Err(FromSqrtError {
-                data: QuadraticSurd::from_rationals(x_y * &y, y, Ratio::from(target.discr)),
-                kind: SqrtErrorKind::Unrepresentable,
-            });
-        }
 
-        // TODO: directly judge sqrt from numerator and denominator
-        let sqrt_delta = match Self::from_sqrt(delta2) {
-            Ok(sq) => sq,
-            Err(e) => {
-                return Err(FromSqrtError {
-                    data: QuadraticSurd::from_rationals(x_y * &y, y, Ratio::from(target.discr)),
-                    kind: e.kind,
-                });
+        // reconstruct the original target when error occurred
+        #[inline]
+        fn reconstruct<T: QuadraticBase>(x_y: Ratio<T>, y: Ratio<T>, r: T, kind: SqrtErrorKind) -> FromSqrtError<QuadraticSurd<T>> where
+        for<'r> &'r T: RefNum<T> { 
+            FromSqrtError {
+                data: QuadraticSurd::from_rationals(x_y * &y, y, Ratio::from(r)),
+                kind: kind,
             }
-        };
-        if !sqrt_delta.is_integer() {
-            return Err(FromSqrtError {
-                data: QuadraticSurd::from_rationals(x_y * &y, y, Ratio::from(target.discr)),
-                kind: SqrtErrorKind::Unrepresentable,
-            });
         }
 
-        let delta2 = match sqrt_delta.to_integer() {
-            Approximation::Exact(v) => v,
-            _ => unreachable!(),
+        if delta2.is_negative() {
+            // this branch happens only when discr is positive
+            return Err(reconstruct(x_y, y, target.discr, SqrtErrorKind::Unrepresentable));
+        }
+        let delta = match Self::from_sqrt(delta2) {
+            Ok(v) => v,
+            Err(e) => return Err(reconstruct(x_y, y, target.discr, e.kind))
+        };
+        let delta = if delta.is_rational() {
+            delta.to_rational().value()
+        } else {
+            return Err(reconstruct(x_y, y, target.discr, SqrtErrorKind::Unrepresentable))
         };
-        let g = x_y - delta2; // XXX: shall we select another root x_y + delta2?
-        let two = T::one() + T::one();
-        let c2 = Ratio::from(two * g.numer() * g.denom()) / y;
-        debug_assert!(c2.is_integer());
-        let c = sqrt(c2.to_integer());
-        debug_assert!(&c * &c == c2.to_integer());
 
-        let (a, b) = g.into();
-        Ok(QuadraticSurd::new(a, b, c, target.discr))
+        // from the equation above, y = 2*a_c*b_c => c = sqrt(2ab/y)
+        // this function find a tuple of integers (a, b, c) that satisfies the equation, and
+        // the output a will be ensured to be non-negative
+        fn get_abc<T: QuadraticBase> (g: Ratio<T>, y: &Ratio<T>) -> Option<(T, T, T)> where
+        for<'r> &'r T: RefNum<T> {
+            let two_ab = (T::one() + T::one()) * g.numer() * g.denom();
+            let d = two_ab.gcd(y.numer());
+            let scale = y.numer() / d;
+            let c2 = two_ab * &scale * &scale * y.denom() / y.numer();
+            let (a, b) = (g.numer() * &scale, g.denom() * scale);
+            if c2.is_negative() {
+                return None;
+            }
+            let c = c2.sqrt();
+            if &c * &c != c2 {
+                None
+            } else {
+                Some((a, b, c))
+            }
+        }
+
+        let ret = if let Some((a, b, c)) = get_abc(&x_y - &delta, &y) {
+            QuadraticSurd::new(a, b, c, target.discr)
+        } else if let Some((a, b, c)) = get_abc(&x_y + delta, &y) {
+            QuadraticSurd::new(a, b, c, target.discr)
+        } else {
+            return Err(reconstruct(x_y, y, target.discr, SqrtErrorKind::Unrepresentable))
+        };
+
+        debug_assert!(!ret.coeffs.0.is_negative());
+        Ok(ret)
     }
 }
 
@@ -1480,15 +1541,18 @@ mod tests {
     pub const N_SQ5: QuadraticSurd<i32> = QuadraticSurd::new_raw(0, -1, 1, 5);
 
     #[test]
-    fn from_rationals_test() {
-        assert_eq!(
-            QuadraticSurd::from_rationals(Ratio::new(1, 2), Ratio::new(1, 2), Ratio::from(5)),
-            PHI
-        );
-        assert_eq!(
-            QuadraticSurd::from_rationals(Ratio::new(-1, 2), Ratio::new(1, 2), Ratio::from(5)),
-            PHI_R
-        );
+    fn creation_test() {
+        let coeffs: (i32, i32, i32, i32) = QuadraticSurd::new(2, 6, 2, 5).into();
+        assert_eq!(coeffs, (1, 3, 1, 5)); // reduce common divisors
+        
+        let coeffs: (i32, i32, i32, i32) = QuadraticSurd::new(2, 1, 2, 18).into();
+        assert_eq!(coeffs, (2, 1, 2, 18)); // 18 is not trivially reducible
+
+        let coeffs: (i32, i32, i32, i32) = QuadraticSurd::new(3, 1, 3, 18).into();
+        assert_eq!(coeffs, (1, 1, 1, 2)); // 18 is reducible with the help of gcd hint
+
+        let coeffs: (i32, i32, i32, i32) = QuadraticSurd::new(3, 1, 3, 9).into();
+        assert_eq!(coeffs, (2, 0, 1, 0)); // 9 is a square number
     }
 
     #[test]
@@ -1510,6 +1574,17 @@ mod tests {
 
     #[test]
     fn from_xxx_test() {
+        // from_rationals
+        assert_eq!(
+            QuadraticSurd::from_rationals(Ratio::new(1, 2), Ratio::new(1, 2), Ratio::from(5)),
+            PHI
+        );
+        assert_eq!(
+            QuadraticSurd::from_rationals(Ratio::new(-1, 2), Ratio::new(1, 2), Ratio::from(5)),
+            PHI_R
+        );
+
+        // from_sqrt
         assert_eq!(
             QuadraticSurd::from_sqrt(5).unwrap(),
             QuadraticSurd::new_raw(0, 1, 1, 5)
@@ -1548,8 +1623,6 @@ mod tests {
         assert_eq!(PHI45_R + Ratio::one(), PHI45);
         assert_eq!(N_PHI + Ratio::one(), N_PHI_R);
         assert_eq!(PHI + PHI_R, SQ5);
-        assert_eq!(PHI45 + PHI_R, SQ5);
-        assert_eq!(PHI + PHI45_R, SQ5);
         assert_eq!(N_PHI + N_PHI_R, N_SQ5);
         assert!((PHI + N_PHI).is_zero());
         assert!((PHI + N_PHI_R).is_one());
@@ -1562,9 +1635,7 @@ mod tests {
         assert_eq!(PHI45 - Ratio::one(), PHI45_R);
         assert_eq!(N_PHI_R - Ratio::one(), N_PHI);
         assert!((PHI - PHI).is_zero());
-        assert!((PHI - PHI45).is_zero());
         assert!((PHI - PHI_R).is_one());
-        assert!((PHI - PHI45_R).is_one());
         assert!((N_PHI_R - N_PHI).is_one());
         assert_eq!(PHI - N_PHI_R, SQ5);
         assert_eq!(N_PHI - PHI_R, -SQ5);
@@ -1575,8 +1646,6 @@ mod tests {
         assert_eq!(PHI * Ratio::one(), PHI);
         assert_eq!(PHI * -Ratio::one(), N_PHI);
         assert!((PHI * PHI_R).is_one());
-        assert!((PHI45 * PHI_R).is_one());
-        assert!((PHI * PHI45_R).is_one());
         assert!((N_PHI * N_PHI_R).is_one());
         assert_eq!(PHI * PHI, PHI_SQ);
         assert_eq!(N_PHI * N_PHI, PHI_SQ);
@@ -1587,8 +1656,6 @@ mod tests {
         assert_eq!(PHI / Ratio::one(), PHI);
         assert_eq!(PHI / -Ratio::one(), N_PHI);
         assert!((PHI / PHI).is_one());
-        assert!((PHI45 / PHI).is_one());
-        assert!((PHI / PHI45).is_one());
         assert!((N_PHI / N_PHI).is_one());
         assert_eq!(PHI / PHI_R, PHI_SQ);
         assert_eq!(N_PHI / N_PHI_R, PHI_SQ);
@@ -1600,10 +1667,8 @@ mod tests {
         assert_eq!(PHI + SQ5 - PHI, SQ5);
         assert_eq!(PHI + 5 - PHI45, QuadraticSurd::from(5));
         assert_eq!(PHI + three_half - PHI45, QuadraticSurd::from(three_half));
-        assert_eq!(PHI + SQ5 - PHI45, SQ5);
         assert_eq!(PHI * 5 / PHI45, QuadraticSurd::from(5));
         assert_eq!(PHI * three_half / PHI45, QuadraticSurd::from(three_half));
-        assert_eq!(PHI * SQ5 / PHI45, SQ5);
 
         // mixed
         assert_eq!(PHI * 2 - 1, SQ5);
@@ -1612,6 +1677,27 @@ mod tests {
         assert_eq!((PHI - Ratio::new(1, 2)) * 2, SQ5);
     }
 
+    #[test]
+    fn arithmic_test_diff_base() {
+        assert_eq!(PHI45 + PHI_R, SQ5);
+        assert_eq!(PHI + PHI45_R, SQ5);
+        assert!((PHI - PHI45).is_zero());
+        assert!((PHI - PHI45_R).is_one());
+        assert!((PHI45 * PHI_R).is_one());
+        assert!((PHI * PHI45_R).is_one());
+        assert!((PHI45 / PHI).is_one());
+        assert!((PHI / PHI45).is_one());
+        assert_eq!(PHI + SQ5 - PHI45, SQ5);
+        assert_eq!(PHI * SQ5 / PHI45, SQ5);
+
+        let a = QuadraticSurd::new(0, 2, 3, 2);
+        let b = QuadraticSurd::new(0, 3, 4, 3);
+        let c = QuadraticSurd::new(0, 1, 2, 6);
+        assert_eq!(a * b, c);
+        assert_eq!(c / a, b);
+        assert_eq!(c / b, a);
+    }
+
     #[test]
     fn conversion_between_contfrac() {
         let cf_phi = ContinuedFraction::<u32>::new(vec![1i32], vec![1], false);
@@ -1646,110 +1732,103 @@ mod tests {
     fn formatting_test() {
         assert_eq!(format!("{}", QuadraticSurd::<i8>::zero()), "0");
         assert_eq!(format!("{}", QuadraticSurd::<i8>::one()), "1");
-        assert_eq!(
-            format!("{}", QuadraticSurd::from_sqrt(5).unwrap()),
-            "√5"
-        );
+        assert_eq!(format!("{}", QuadraticSurd::from_sqrt(5).unwrap()), "√5");
 
         assert_eq!(format!("{}", QuadraticSurd::new(1, 1, 1, 5)), "1+√5");
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(1, -1, 1, 5)),
-            "1-√5"
-        );
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(-1, 1, 1, 5)),
-            "-1+√5"
-        );
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(-1, -1, 1, 5)),
-            "-1-√5"
-        );
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(1, 2, 1, 5)),
-            "1+2√5"
-        );
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(1, -2, 1, 5)),
-            "1-2√5"
-        );
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(-1, 2, 1, 5)),
-            "-1+2√5"
-        );
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(-1, -2, 1, 5)),
-            "-1-2√5"
-        );
+        assert_eq!(format!("{}", QuadraticSurd::new(1, -1, 1, 5)), "1-√5");
+        assert_eq!(format!("{}", QuadraticSurd::new(-1, 1, 1, 5)), "-1+√5");
+        assert_eq!(format!("{}", QuadraticSurd::new(-1, -1, 1, 5)), "-1-√5");
+        assert_eq!(format!("{}", QuadraticSurd::new(1, 2, 1, 5)), "1+2√5");
+        assert_eq!(format!("{}", QuadraticSurd::new(1, -2, 1, 5)), "1-2√5");
+        assert_eq!(format!("{}", QuadraticSurd::new(-1, 2, 1, 5)), "-1+2√5");
+        assert_eq!(format!("{}", QuadraticSurd::new(-1, -2, 1, 5)), "-1-2√5");
         assert_eq!(format!("{}", QuadraticSurd::new(1, 0, 2, 5)), "1/2");
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(-1, 0, 2, 5)),
-            "-1/2"
-        );
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(1, 1, 2, 5)),
-            "(1+√5)/2"
-        );
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(1, -1, 2, 5)),
-            "(1-√5)/2"
-        );
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(-1, 1, 2, 5)),
-            "(-1+√5)/2"
-        );
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(-1, -1, 2, 5)),
-            "(-1-√5)/2"
-        );
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(1, 2, 2, 5)),
-            "(1+2√5)/2"
-        );
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(1, -2, 2, 5)),
-            "(1-2√5)/2"
-        );
-        assert_eq!(
-            format!("{}", QuadraticSurd::new(-1, 2, 2, 5)),
-            "(-1+2√5)/2"
-        );
+        assert_eq!(format!("{}", QuadraticSurd::new(-1, 0, 2, 5)), "-1/2");
+        assert_eq!(format!("{}", QuadraticSurd::new(1, 1, 2, 5)), "(1+√5)/2");
+        assert_eq!(format!("{}", QuadraticSurd::new(1, -1, 2, 5)), "(1-√5)/2");
+        assert_eq!(format!("{}", QuadraticSurd::new(-1, 1, 2, 5)), "(-1+√5)/2");
+        assert_eq!(format!("{}", QuadraticSurd::new(-1, -1, 2, 5)), "(-1-√5)/2");
+        assert_eq!(format!("{}", QuadraticSurd::new(1, 2, 2, 5)), "(1+2√5)/2");
+        assert_eq!(format!("{}", QuadraticSurd::new(1, -2, 2, 5)), "(1-2√5)/2");
+        assert_eq!(format!("{}", QuadraticSurd::new(-1, 2, 2, 5)), "(-1+2√5)/2");
         assert_eq!(
             format!("{}", QuadraticSurd::new(-1, -2, 2, 5)),
             "(-1-2√5)/2"
         );
     }
+
+    #[test]
+    fn trunc_frac_test() {
+        assert_eq!(PHI.trunc_ref(), QuadraticSurd::from(1));
+        assert_eq!(PHI.fract_ref(), PHI_R);
+        assert_eq!(PHI_R.trunc_ref(), QuadraticSurd::zero());
+        assert_eq!(PHI_R.fract_ref(), PHI_R);
+        assert_eq!(N_PHI.trunc_ref(), QuadraticSurd::from(-1));
+        assert_eq!(N_PHI.fract_ref(), N_PHI_R);
+        assert_eq!(N_PHI_R.trunc_ref(), QuadraticSurd::from(0));
+        assert_eq!(N_PHI_R.fract_ref(), N_PHI_R);
+        assert_eq!(PHI_SQ.trunc_ref(), QuadraticSurd::from(2));
+        assert_eq!(PHI_SQ.fract_ref(), PHI_R);
+        assert_eq!(PHI45.trunc_ref(), QuadraticSurd::from(1));
+        assert_eq!(PHI45.fract_ref(), PHI_R);
+        assert_eq!(PHI45_R.trunc_ref(), QuadraticSurd::from(0));
+        assert_eq!(PHI45_R.fract_ref(), PHI_R);
+        assert_eq!(SQ5.trunc_ref(), QuadraticSurd::from(2));
+        assert_eq!(SQ5.fract_ref(), QuadraticSurd::new(-2, 1, 1, 5));
+        assert_eq!(N_SQ5.trunc_ref(), QuadraticSurd::from(-2));
+        assert_eq!(N_SQ5.fract_ref(), QuadraticSurd::new(2, -1, 1, 5));
+    }
+
+    #[test]
+    fn from_sqrt_test() {
+        assert_eq!(
+            QuadraticSurd::from_sqrt(5i32).unwrap(),
+            QuadraticSurd::new_raw(0, 1, 1, 5)
+        );
+        assert_eq!(
+            QuadraticSurd::from_sqrt(QuadraticSurd::new(3i32, 2, 1, 2)).unwrap(),
+            QuadraticSurd::new_raw(1, 1, 1, 2)
+        );
+
+        #[cfg(not(feature = "complex"))]
+        {
+            let err = QuadraticSurd::from_sqrt(-2i32).unwrap_err();
+            assert_eq!(
+                err.kind,
+                SqrtErrorKind::Complex
+            );
+            assert_eq!(err.data, -2);
+            let err = QuadraticSurd::from_sqrt(Ratio::new(-1i32, 2)).unwrap_err();
+            assert_eq!(
+                err.kind,
+                SqrtErrorKind::Complex
+            );
+            assert_eq!(err.data, Ratio::new(-1, 2));
+            let surd = QuadraticSurd::new(-2i32, 1, 1, 2);
+            let err = QuadraticSurd::from_sqrt(surd).unwrap_err();
+            assert_eq!(
+                err.kind,
+                SqrtErrorKind::Complex
+            );
+            assert_eq!(err.data, surd);
+        }
+    }
 }
 
+#[cfg(feature = "complex")]
 #[cfg(test)]
 mod complex_tests {
     use super::*;
 
-    #[cfg(not(feature = "complex"))]
+    pub const OMEGA: QuadraticSurd<i32> = QuadraticSurd::new_raw(-1, 1, 2, -3); // -0.5 + 0.866i
+    pub const OMEGA3: QuadraticSurd<i32> = QuadraticSurd::new_raw(-3, 3, 2, -3); // -1.5 + 2.598i
+
     #[test]
-    fn from_sqrt_test() {
-        assert_eq!(
-            QuadraticSurd::from_sqrt(5).unwrap(),
-            QuadraticSurd::new_raw(0, 1, 1, 5)
-        );
-        assert_eq!(
-            QuadraticSurd::from_sqrt(-2).unwrap_err().kind,
-            SqrtErrorKind::Complex
-        );
-        assert_eq!(
-            QuadraticSurd::from_sqrt(Ratio::new(-1, 2))
-                .unwrap_err()
-                .kind,
-            SqrtErrorKind::Complex
-        );
-        assert_eq!(
-            QuadraticSurd::from_sqrt(QuadraticSurd::from(-1))
-                .unwrap_err()
-                .kind,
-            SqrtErrorKind::Complex
-        );
+    fn trunc_frac_test() {
+        assert_eq!(OMEGA.trunc_ref(), QuadraticSurd::zero());
+        assert_eq!(OMEGA3.trunc_ref(), QuadraticSurd::new(-1, 2, 1, -1));
     }
 
-    #[cfg(feature = "complex")]
     #[test]
     fn from_sqrt_test() {
         assert!(QuadraticSurd::from_sqrt(-2i32).is_ok());
@@ -1761,85 +1840,77 @@ mod complex_tests {
         let i = QuadraticSurd::from_sqrt(QuadraticSurd::from(-1i32)).unwrap();
         assert_eq!(sqhalfi * 2, sq2i);
         assert_eq!(sq2i * i, QuadraticSurd::from_sqrt(2i32).unwrap());
+        
+        assert_eq!(
+            QuadraticSurd::from_sqrt(QuadraticSurd::new(-3i32, 4, 1, -1)).unwrap(),
+            QuadraticSurd::new_raw(1, 2, 1, -1)
+        );
     }
 
-    #[cfg(feature = "complex")]
     #[test]
     fn formatting_test() {
-        assert_eq!(format!("{}", QuadraticSurd::<i32>::zero()), "0");
-        assert_eq!(format!("{}", QuadraticSurd::<i32>::one()), "1");
+        // test trivial complex cases
+        assert_eq!(format!("{:#}", QuadraticSurd::from_sqrt(-1).unwrap()), "i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, 1, 1, -1)), "1+i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, -1, 1, -1)), "1-i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(-1, 1, 1, -1)), "-1+i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(-1, -1, 1, -1)), "-1-i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, 2, 1, -1)), "1+2i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, -2, 1, -1)), "1-2i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(-1, 2, 1, -1)), "-1+2i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(-1, -2, 1, -1)), "-1-2i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, 0, 2, -1)), "1/2");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(-1, 0, 2, -1)), "-1/2");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, 1, 2, -1)), "(1+i)/2");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, -1, 2, -1)), "(1-i)/2");
         assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::from_sqrt(-1).unwrap()),
-            "i"
-        );
-
-        assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(1, 1, 1, -1)),
-            "1+i"
-        );
-        assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(1, -1, 1, -1)),
-            "1-i"
-        );
-        assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(-1, 1, 1, -1)),
-            "-1+i"
-        );
-        assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(-1, -1, 1, -1)),
-            "-1-i"
-        );
-        assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(1, 2, 1, -1)),
-            "1+2i"
-        );
-        assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(1, -2, 1, -1)),
-            "1-2i"
-        );
-        assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(-1, 2, 1, -1)),
-            "-1+2i"
-        );
-        assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(-1, -2, 1, -1)),
-            "-1-2i"
-        );
-        assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(1, 0, 2, -1)),
-            "1/2"
-        );
-        assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(-1, 0, 2, -1)),
-            "-1/2"
-        );
-        assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(1, 1, 2, -1)),
-            "(1+i)/2"
-        );
-        assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(1, -1, 2, -1)),
-            "(1-i)/2"
-        );
-        assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(-1, 1, 2, -1)),
+            format!("{:#}", QuadraticSurd::new(-1, 1, 2, -1)),
             "(-1+i)/2"
         );
         assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(-1, -1, 2, -1)),
+            format!("{:#}", QuadraticSurd::new(-1, -1, 2, -1)),
             "(-1-i)/2"
         );
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, 2, 2, -1)), "(1+2i)/2");
         assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(1, 2, 2, -1)),
-            "(1+2i)/2"
-        );
-        assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(1, -2, 2, -1)),
+            format!("{:#}", QuadraticSurd::new(1, -2, 2, -1)),
             "(1-2i)/2"
         );
         assert_eq!(
-            format!("{:#}", QuadraticSurd::<i32>::new(-1, 2, 2, -1)),
+            format!("{:#}", QuadraticSurd::new(-1, 2, 2, -1)),
             "(-1+2i)/2"
         );
+
+        // test non-trivial complex cases
+        assert_eq!(format!("{:#}", QuadraticSurd::from_sqrt(-3).unwrap()), "√3i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, 1, 1, -3)), "1+√3i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, -1, 1, -3)), "1-√3i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(-1, 1, 1, -3)), "-1+√3i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(-1, -1, 1, -3)), "-1-√3i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, 2, 1, -3)), "1+2√3i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, -2, 1, -3)), "1-2√3i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(-1, 2, 1, -3)), "-1+2√3i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(-1, -2, 1, -3)), "-1-2√3i");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, 0, 2, -3)), "1/2");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(-1, 0, 2, -3)), "-1/2");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, 1, 2, -3)), "(1+√3i)/2");
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, -1, 2, -3)), "(1-√3i)/2");
+        assert_eq!(
+            format!("{:#}", QuadraticSurd::new(-1, 1, 2, -3)),
+            "(-1+√3i)/2"
+        );
+        assert_eq!(
+            format!("{:#}", QuadraticSurd::new(-1, -1, 2, -3)),
+            "(-1-√3i)/2"
+        );
+        assert_eq!(format!("{:#}", QuadraticSurd::new(1, 2, 2, -3)), "(1+2√3i)/2");
+        assert_eq!(
+            format!("{:#}", QuadraticSurd::new(1, -2, 2, -3)),
+            "(1-2√3i)/2"
+        );
+        assert_eq!(
+            format!("{:#}", QuadraticSurd::new(-1, 2, 2, -3)),
+            "(-1+2√3i)/2"
+        );
     }
 }
